﻿
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

// To compute the eigenvalues and eigenvectors of a symmetric matrix by Jakobi transformations


namespace WindowsFormsApplication1
{

     public partial class MainWin : Form
    {
        const int MaxOrder = 4;
        public TextBox[,] tbm;
        public TextBox[] tbs;
        public TextBox[] tbx;
        //TMatrix r;

        Stopwatch timer = new Stopwatch();

        public double[,] a = new double[MaxOrder, MaxOrder];
        public double[,] r = new double[MaxOrder, MaxOrder];
        public double[,] v = new double[MaxOrder, MaxOrder];

        public double[,] MultMatrixRight(double[,] a, double[,] m, int p, int q, int size)
        {
            int i, j, k;
            double[,] al = new double[size, 2];
            for (i = 0; i < size; i++)
            {
                al[i, 0] = 0.0;
                al[i, 1] = 0.0;
                for (k = 0; k < size; k++)
                {
                    al[i, 0] = al[i, 0] + (a[i, k] * m[k, p]);
                    al[i, 1] = al[i, 1] + (a[i, k] * m[k, q]);
                }
            }
            for (i = 0; i < size; i++)
            {
                a[i, p] = al[i, 0];
                a[i, q] = al[i, 1];
            }
            return a;
        }


        public double[,] MultTranspMatrixLeft(double[,] a, double[,] m, int p, int q, int size)
        {
            int j, k;
            double[,] al = new double[2, size];
            for (j = 0; j < size; j++)
            {
                al[0, j] = 0.0;
                al[1, j] = 0.0;
                for (k = 0; k < size; k++)
                {
                    al[0, j] = al[0, j] + (m[k, p] * a[k, j]);
                    al[1, j] = al[1, j] + (m[k, q] * a[k, j]);
                }
            }
            for (j = 0; j < size; j++)
            {
                a[p, j] = al[0, j];
                a[q, j] = al[1, j];
            }
            return a;
        }


        public void CalcRMatrix(double[,] a, int p, int q, int size)
        {
            int i, j;
            double phi;
            double t;
            phi = (a[q, q] - a[p, p]) / 2 / a[q, p];
            if (phi != 0)
            {
                if (phi > 0)
                    t = 1/(phi + Math.Sqrt(phi * phi + 1.0));
                else
                    t = 1/(phi - Math.Sqrt(phi * phi + 1.0));
            }
            else
                t = 1.0;
            for (i = 0; i < size; i++)
            {
                for (j = 0; j < size; j++)
                {
                    r[i, j] = 0.0;
                }
                r[i, i] = 1.0;
            }
            r[p, p] = 1 / Math.Sqrt(t * t + 1.0);
            r[p, q] = t * r[p, p];
            r[q, q] = r[p, p];
            r[q, p] = -r[p, q];
        }


        public void RunJacobi(int maxOrder)
        {
            int i, k, count = 0;
            double delta = 10;
            while ((delta > 1.0E-50) && (count < 50))
            {
                delta = 0;
                count++;
                for (i = 0; i < maxOrder; i++)
                {
                    for (k = i + 1; k < maxOrder; k++)
                    {
                        if (a[k, i] != 0.0)
                        {
                            CalcRMatrix(a, i, k, maxOrder);
                            a = MultTranspMatrixLeft(a, r, i, k, maxOrder);
                            a = MultMatrixRight(a, r, i, k, maxOrder);
                            v = MultMatrixRight(v, r, i, k, maxOrder);
                        }
                    }
                    if(i< maxOrder-1)
                     delta = delta + Math.Abs(a[i+1, i]);
                }
            }    
        }

        public MainWin()
        {
            int i, k;
         
            DataGridViewCell cell;

            InitializeComponent();

            GMatrixA.ColumnCount = MaxOrder;
            GMatrixA.RowCount = MaxOrder;
            dGEigenVectors.ColumnCount = MaxOrder;
            dGEigenVectors.RowCount = MaxOrder;
            for (i = 0; i < GMatrixA.ColumnCount; i++)
            {
                GMatrixA.Columns[i].Width = 160;
                dGEigenVectors.Columns[i].Width = 160;
            }
            GSearched.ColumnCount = MaxOrder;
            GSearched.RowCount = 1;
          
            // Sample matrix
            a[0, 0] = 20.0;
            a[0, 1] = -7.0;
            a[0, 2] = 3.0;
            a[0, 3] = -2.0; 
         
            a[1, 0] = -7.0;
            a[1, 1] = 5.0;
            a[1, 2] = 1.0;
            a[1, 3] = 4.0;
             
            a[2, 0] = 3.0;
            a[2, 1] = 1.0;
            a[2, 2] = 3.0;
            a[2, 3] = 1.0;

            a[3, 0] = -2.0;
            a[3, 1] = 4.0;
            a[3, 2] = 1.0;
            a[3, 3] = 2.0;
            // real Eigenvalues:   { -1,160949791926147 ;  1,173048870316898 ;  6,460514719957084 ;  23,527386201652163 }

          

            for (i = 0; i < MaxOrder; i++)
            {
                for (k = 0; k < MaxOrder; k++)
                {
                    cell = GMatrixA[i, k];  // get cell to access
                    cell.Value = Convert.ToString(a[k, i]);
                    v[k, i] = 0.0;
                }
                v[i, i] = 1.0;
            }
        }

        private void button1_MouseClick(object sender, MouseEventArgs e)
        {
            int i, k;
            double[] buf = new double[MaxOrder];
            DataGridViewCell cell;

            for (i = 0; i < MaxOrder; i++)
            {
                for (k = 0; k < MaxOrder; k++)
                {
                    cell = GMatrixA[i, k];  // get cell to access
                    a[k, i] = Convert.ToDouble(cell.Value);
                    v[k, i] = 0.0;
                }
                v[i, i] = 1.0;
            }

            RunJacobi(MaxOrder);

            for (i = 0; i < MaxOrder; i++)
            {
                for (k = 0; k < MaxOrder; k++)
                {
                    cell = GMatrixA[i, k];  // get cell to access
                    cell.Value = Convert.ToString(a[k, i]);

                    cell = dGEigenVectors[i, k];
                    cell.Value = Convert.ToString(v[k, i]);

                }
                cell = GSearched[i, 0];
                cell.Value = Convert.ToString(a[i, i]);
            }     
        }    
    }
}
